﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MidiKeyboard
{
    public partial class MidiKeyboard : Form
    {
        private playSound playSnd = new playSound();
        private List<Key> keys = new List<Key>();
        private List<Instrument> instruments = new List<Instrument>();
        private List<Octave> octaves = new List<Octave>();
        private uint selOctaveIndex = 0;
        private List<string> files = new List<string>();
        char[] splitChar = new char[3];
        private MidiOut midi = new MidiOut();
        private bool[] FirstKey = new bool[24];

        public MidiKeyboard()
        {
            InitializeComponent();
        }

        private void MidiKeyboard_Load(object sender, EventArgs e)
        {
            // only for file-string splitting
            splitChar[0] = '\\';
            splitChar[1] = '\\';
            splitChar[2] = (char)0;

            // create midi keys
            keys.Add(new Key((int)Keys.A, 0));
            keys.Add(new Key((int)Keys.W, 1));
            keys.Add(new Key((int)Keys.S, 2));
            keys.Add(new Key((int)Keys.E, 3));
            keys.Add(new Key((int)Keys.D, 4));
            keys.Add(new Key((int)Keys.F, 5));
            keys.Add(new Key((int)Keys.R, 6));
            keys.Add(new Key((int)Keys.G, 7));
            keys.Add(new Key((int)Keys.T, 8));
            keys.Add(new Key((int)Keys.H, 9));
            keys.Add(new Key((int)Keys.Z, 10));
            keys.Add(new Key((int)Keys.J, 11));
            keys.Add(new Key((int)(Keys.A | Keys.Shift), 12));
            keys.Add(new Key((int)(Keys.W | Keys.Shift), 13));
            keys.Add(new Key((int)(Keys.S | Keys.Shift), 14));
            keys.Add(new Key((int)(Keys.E | Keys.Shift), 15));
            keys.Add(new Key((int)(Keys.D | Keys.Shift), 16));
            keys.Add(new Key((int)(Keys.F | Keys.Shift), 17));
            keys.Add(new Key((int)(Keys.R | Keys.Shift), 18));
            keys.Add(new Key((int)(Keys.G | Keys.Shift), 19));
            keys.Add(new Key((int)(Keys.T | Keys.Shift), 20));
            keys.Add(new Key((int)(Keys.H | Keys.Shift), 21));
            keys.Add(new Key((int)(Keys.Z | Keys.Shift), 22));
            keys.Add(new Key((int)(Keys.J | Keys.Shift), 23));
            

            // create instruments
            instruments.Add(new Instrument("Grand Piano", 0));
            instruments.Add(new Instrument("Electric Piano", 4));
            instruments.Add(new Instrument("Xylophone", 13));
            instruments.Add(new Instrument("Rock Organ", 18));
            instruments.Add(new Instrument("Accoridion", 21));
            instruments.Add(new Instrument("Electric Guitar", 26));
            instruments.Add(new Instrument("Acoustic Bass", 32));
            instruments.Add(new Instrument("Violin", 40));
            instruments.Add(new Instrument("Trumpet", 56));
            instruments.Add(new Instrument("Alto Sax", 65));
            instruments.Add(new Instrument("English Horn", 69));
            instruments.Add(new Instrument("Clarinet", 71));

            foreach (Instrument tmpInstr in instruments)
            {
                listBox_instrument.Items.Add(tmpInstr.titel);
            }
            listBox_instrument.SelectedIndex = 0;


            // create octaves
            octaves.Add(new Octave("subc. oct., contra oct.", 12));
            octaves.Add(new Octave("big octave, small octave", 36));
            octaves.Add(new Octave("'octave, ''octave", 60));
            octaves.Add(new Octave("'''octave, ''''octave", 84));
            octaves.Add(new Octave("'''''octave, ''''''octave", 108));

            foreach (Octave tmpOctave in octaves)
            {
                listBox_octave.Items.Add(tmpOctave.titel);
            }
            listBox_octave.SelectedIndex = 2;
            selOctaveIndex = octaves[listBox_octave.SelectedIndex].startNr;


            // open midi interface
            if (midi.Open() == 0)
            {
                // if midi device can't be opened -> program will end
                MessageBox.Show("midi device can not be opened", "MIDI Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                midi.Close();
            }
            uint instrNr = instruments[listBox_instrument.SelectedIndex].number;
            midi.Instrument(instrNr);
        }

        private void MidiKeyboard_FormClosing(object sender, FormClosingEventArgs e)
        {
            midi.Close();
        }

        private void button_accept_Click(object sender, EventArgs e)
        {
            selOctaveIndex = octaves[listBox_octave.SelectedIndex].startNr;

            uint instrNr = instruments[listBox_instrument.SelectedIndex].number;
            midi.Instrument(instrNr);
        }

        #region midi_button_control
        private void button_C1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 0);
        }
        private void button_C1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 0);
        }

        private void button_Cis1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 1);
        }
        private void button_Cis1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 1);
        }

        private void button_D1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 2);
        }
        private void button_D1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 2);
        }

        private void button_Dis1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 3);
        }
        private void button_Dis1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 3);
        }

        private void button_E1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 4);
        }
        private void button_E1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 4);
        }

        private void button_F1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 5);
        }
        private void button_F1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 5);
        }

        private void button_Fis1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 6);
        }
        private void button_Fis1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 6);
        }

        private void button_G1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 7);
        }
        private void button_G1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 7);
        }

        private void button_Gis1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 8);
        }
        private void button_Gis1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 8);
        }

        private void button_A1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 9);
        }
        private void button_A1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 9);
        }

        private void button_Ais1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 10);
        }
        private void button_Ais1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 10);
        }

        private void button_H1_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 11);
        }
        private void button_H1_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 11);
        }

        private void button_C2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 12);
        }
        private void button_C2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 12);
        }

        private void button_Cis2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 13);
        }
        private void button_Cis2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 13);
        }

        private void button_D2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 14);
        }
        private void button_D2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 14);
        }

        private void button_Dis2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 15);
        }
        private void button_Dis2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 15);
        }

        private void button_E2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 16);
        }
        private void button_E2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 16);
        }

        private void button_F2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 17);
        }
        private void button_F2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 17);
        }

        private void button_Fis2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 18);
        }
        private void button_Fis2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 18);
        }

        private void button_G2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 19);
        }
        private void button_G2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 19);
        }

        private void button_Gis2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 20);
        }
        private void button_Gis2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 20);
        }

        private void button_A2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 21);
        }
        private void button_A2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 21);
        }

        private void button_Ais2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 22);
        }
        private void button_Ais2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 22);
        }

        private void button_H2_MouseDown(object sender, MouseEventArgs e)
        {
            midi.Play(selOctaveIndex + 23);
        }
        private void button_H2_MouseUp(object sender, MouseEventArgs e)
        {
            midi.Stop(selOctaveIndex + 23);
        }
        #endregion

        #region midi_keyboard_control
        private void MidiKeyboard_KeyDown(object sender, KeyEventArgs e)
        {
            foreach (Key tmpKey in keys)
            {
                if ((e.KeyData == (Keys)tmpKey.keyNr) && tmpKey.firstDown)
                {
                    tmpKey.firstDown = false;
                    midi.Play(selOctaveIndex + tmpKey.noteNr);
                }
            }
        }

        private void MidiKeyboard_KeyUp(object sender, KeyEventArgs e)
        {
            foreach (Key tmpKey in keys)
            {
                if (e.KeyData == (Keys)tmpKey.keyNr)
                {
                    tmpKey.firstDown = true;
                    midi.Stop(selOctaveIndex + tmpKey.noteNr);
                }
            }
        }
        #endregion

        private void button_Dir_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = @"C:\";
            ofd.Title = "Browse for background sounds.";
            ofd.Filter = "Wav Files (*.wav) | *.wav";
            ofd.Multiselect = true;

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                files.Clear();
                listBox_wavs.Items.Clear();
                textBox_playedWav.Text = "                 --------------------";

                foreach (string file in ofd.FileNames)
                {
                    files.Add(file);

                    string[] split = file.Split(splitChar);
                    listBox_wavs.Items.Add(split[split.Length-1]);
                }
            }
        }

        private void listBox_wavs_SelectedValueChanged(object sender, EventArgs e)
        {
            textBox_playedWav.Text = (string)listBox_wavs.SelectedItem;

            if (playSnd.playing)
            {
                playSnd.play((string)listBox_wavs.SelectedItem);
            }
        }

        private void button_play_Click(object sender, EventArgs e)
        {
            playSnd.play((string)listBox_wavs.SelectedItem);
        }

        private void button_stop_Click(object sender, EventArgs e)
        {
            playSnd.stop();
        }
    }
}
