﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace MidiKeyboard
{
    // class for playing midi tones
    public class MidiOut
    {
        // dll function definitions
        [DllImport("winmm.dll", SetLastError = true,
            CallingConvention = CallingConvention.Winapi)]
        private static extern int midiOutOpen(ref int handle,
                                              int deviceID,
                                              IntPtr proc,
                                              int instance,
                                              int flags);

        [DllImport("winmm.dll", SetLastError = true,
            CallingConvention = CallingConvention.Winapi)]
        private static extern int midiOutClose(int handle);

        [DllImport("winmm.dll", SetLastError = true,
            CallingConvention = CallingConvention.Winapi)]
        private static extern int midiOutShortMsg(int handle,
                                                  uint message);

        [DllImport("winmm.dll", SetLastError = true,
            CallingConvention = CallingConvention.Winapi)]
        public static extern uint midiOutGetNumDevs();


        // member for midi handler
        private int hMidi = 0;
        
        // opens the midi device
        public int Open()
        {
            try
            {
                int port = (int)midiOutGetNumDevs() - 1;
                midiOutOpen(ref hMidi, port, IntPtr.Zero, 0, 0);
            }
            catch
            {
                int err = Marshal.GetLastWin32Error();
                if (err != 0)
                {
                    MessageBox.Show(err.ToString(), "MIDI Error",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            return hMidi;
        }

        // closes the midi device
        public void Close()
        {
            if (hMidi != 0)
            {
                try
                {
                    midiOutClose(hMidi);
                }
                catch
                {
                    int err = Marshal.GetLastWin32Error();
                    if (err != 0)
                    {
                        MessageBox.Show(err.ToString(), "MIDI Error",
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        // selects an instrument
        public void Instrument(uint instrument)
        {
            if (hMidi != 0)
            {
                try
                {
                    uint message = 0;
                    message += (uint)(0xC0 << 0);       // choose instrument
                    message += (uint)(instrument << 8); // instrument
                    midiOutShortMsg(hMidi, message);
                }
                catch
                {
                    int err = Marshal.GetLastWin32Error();
                    if (err != 0)
                    {
                        MessageBox.Show(err.ToString(), "MIDI Error",
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        // plays a tone
        public void Play(uint tone)
        {
            if (hMidi != 0)
            {
                try
                {
                    uint message = 0;
                    message += (uint)(0x90 << 0);       // note on
                    message += (uint)(tone << 8);       // tone number
                    message += (uint)(127 << 16);       // volume
                    midiOutShortMsg(hMidi, message);
                }
                catch
                {
                    int err = Marshal.GetLastWin32Error();
                    if (err != 0)
                    {
                        MessageBox.Show(err.ToString(), "MIDI Error",
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        // stops playing a tone
        public void Stop(uint tone)
        {
            if (hMidi != 0)
            {
                try
                {
                    uint message = 0;
                    message += (uint)(0x80 << 0);       // note off
                    message += (uint)(tone << 8);       // tone number
                    message += (uint)(127 << 16);       // volume
                    midiOutShortMsg(hMidi, message);
                }
                catch
                {
                    int err = Marshal.GetLastWin32Error();
                    if (err != 0)
                    {
                        MessageBox.Show(err.ToString(), "MIDI Error",
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }
    }
}
